module.exports['menu_seperator.png.http'] = [
  {
    type: 'file',
    name: 'image',
    filename: 'menu_separator.png',
    fixture: 'menu_separator.png',
    sha1: 'c845ca3ea794be298f2a1b79769b71939eaf4e54',
    size: 931
  }
];

module.exports['beta-sticker-1.png.http'] = [
  {
    type: 'file',
    name: 'sticker',
    filename: 'beta-sticker-1.png',
    fixture: 'beta-sticker-1.png',
    sha1: '6abbcffd12b4ada5a6a084fe9e4584f846331bc4',
    size: 1660
  }
];

module.exports['blank.gif.http'] = [
  {
    type: 'file',
    name: 'file',
    filename: 'blank.gif',
    fixture: 'blank.gif',
    sha1: 'a1fdee122b95748d81cee426d717c05b5174fe96',
    size: 49
  }
];

module.exports['binaryfile.tar.gz.http'] = [
  {
    type: 'file',
    name: 'file',
    filename: 'binaryfile.tar.gz',
    fixture: 'binaryfile.tar.gz',
    sha1: 'cfabe13b348e5e69287d677860880c52a69d2155',
    size: 301
  }
];

module.exports['plain.txt.http'] = [
  {
    type: 'file',
    name: 'file',
    filename: 'plain.txt',
    fixture: 'plain.txt',
    sha1: 'b31d07bac24ac32734de88b3687dddb10e976872',
    size: 23
  }
];
