module.exports['crlf.http'] = [
  {type: 'file', name: 'upload', filename: 'plain.txt', fixture: 'plain.txt',
  sha1: 'b31d07bac24ac32734de88b3687dddb10e976872'}
];

module.exports['preamble.http'] = [
  {type: 'file', name: 'upload', filename: 'plain.txt', fixture: 'plain.txt',
  sha1: 'b31d07bac24ac32734de88b3687dddb10e976872'}
];
