module.exports['charset.http'] = [
  {
    type: 'file',
    name: 'file',
    filename: 'plain.txt',
    fixture: 'plain.txt',
    sha1: 'b31d07bac24ac32734de88b3687dddb10e976872',
    size: 23
  }
];

module.exports['charset-last.http'] = [
  {
    type: 'file',
    name: 'file',
    filename: 'plain.txt',
    fixture: 'plain.txt',
    sha1: 'b31d07bac24ac32734de88b3687dddb10e976872',
    size: 23
  }
];

module.exports['custom.http'] = [
  {
    type: 'file',
    name: 'file',
    filename: 'plain.txt',
    fixture: 'plain.txt',
    sha1: 'b31d07bac24ac32734de88b3687dddb10e976872',
    size: 23
  }
];


// to test the regexp modification : it should accepts equal sign in boundary
module.exports['custom-equal-sign.http'] = [
  {
    type: 'file',
    name: 'file',
    filename: 'plain.txt',
    fixture: 'plain.txt',
    size: 24
  }
];
